@@ -3,19 +3,19 @@ import collections
 import grpc
 
 from ddtrace import config
-from ddtrace.compat import stringify
-from ddtrace.compat import to_unicode
 from ddtrace.ext import SpanTypes
 from ddtrace.ext import errors
+from ddtrace.internal.compat import stringify
+from ddtrace.internal.compat import to_unicode
 from ddtrace.vendor import wrapt
 
 from . import constants
+from . import utils
 from .. import trace_utils
 from ...constants import ANALYTICS_SAMPLE_RATE_KEY
 from ...constants import SPAN_MEASURED_KEY
 from ...internal.logger import get_logger
 from ...propagation.http import HTTPPropagator
-from .utils import set_grpc_method_meta
 
 
 log = get_logger(__name__)
@@ -118,6 +118,9 @@ class _WrappedResponseCallFuture(wrapt.ObjectProxy):
     def __init__(self, wrapped, span):
         super(_WrappedResponseCallFuture, self).__init__(wrapped)
         self._span = span
+        # Registers callback on the _MultiThreadedRendezvous future to finish
+        # span in case StopIteration is never raised but RPC is terminated
+        _handle_response(self._span, self.__wrapped__)
 
     def __iter__(self):
         return self
@@ -133,8 +136,7 @@ class _WrappedResponseCallFuture(wrapt.ObjectProxy):
         try:
             return next(self.__wrapped__)
         except StopIteration:
-            # at end of iteration handle response status from wrapped future
-            _handle_response(self._span, self.__wrapped__)
+            # Callback will handle span finishing
             raise
         except grpc.RpcError as rpc_error:
             # DEV: grpcio<1.18.0 grpc.RpcError is raised rather than returned as response
@@ -177,10 +179,8 @@ class _ClientInterceptor(
         )
         span.set_tag(SPAN_MEASURED_KEY)
 
-        set_grpc_method_meta(span, client_call_details.method, method_kind)
-        span._set_str_tag(constants.GRPC_HOST_KEY, self._host)
-        if self._port:
-            span._set_str_tag(constants.GRPC_PORT_KEY, str(self._port))
+        utils.set_grpc_method_meta(span, client_call_details.method, method_kind)
+        utils.set_grpc_client_meta(span, self._host, self._port)
         span._set_str_tag(constants.GRPC_SPAN_KIND_KEY, constants.GRPC_SPAN_KIND_VALUE_CLIENT)
 
         sample_rate = config.grpc.get_analytics_sample_rate()
